#! /usr/bin/env python3
#Author: Martin Kacmarcik
#License: MIT
#For my Diploma thesis at Faculty of Electrical Engineering -- Brno, University of Technology

import socket
import sys

def testPortAvailability(hostname,port):
    try:
        server_ip  = socket.gethostbyname(hostname)
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.settimeout(1)
        result = sock.connect_ex((server_ip, port))
        if result == 0:
            return True
        else:
            return False
        sock.close()
    except KeyboardInterrupt:
        print("You pressed Ctrl+C")
        sys.exit(96)
    except socket.gaierror:
        return 98
    except socket.error:
        return 97



import folium
import csv
from folium.plugins import MarkerCluster


def main():
    """
        Creates a map with ICMP responses for all selected nodes.\n
        :return: map_icmp.html file
        """
    map_icmp = folium.Map(location=[45.523, -122.675],
                          zoom_start=2)

    with open('lib/base_data.txt') as tsv:
        for row in csv.reader(tsv, delimiter='\t'):
            name = row[0]
            try:
                x = float(row[1])
                y = float(row[2])
                print(" %s " % name)
                folium.Marker([x, y], popup=name).add_to(map_icmp)
            except ValueError:
                pass

    map_icmp.save('map_icmp.html')


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import xmlrpc.client
import geocoder
import codecs
import argparse
import sys
import os
import socket
import traceback
from time import sleep, asctime, localtime, time

plc_host = "www.planet-lab.eu"

auth = {'AuthMethod': 'password',
        'AuthString': '',
        'Username': ''
        }

arg = {"path": "",
       "id": 1,
       "start_id": 1,
       "quiet": False}

api_url = "https://%s:443/PLCAPI/" % plc_host
plc_api = xmlrpc.client.ServerProxy(api_url, allow_none=True)

locations = {}

continents = {
    "AD": "EU", "AE": "AS", "AF": "AS", "AG": "NA", "AI": "NA", "AL": "EU", "AM": "AS", "AN": "NA", "AO": "AF",
    "AQ": "AQ", "AR": "SA", "AS": "OC", "AT": "EU", "AU": "OC", "AW": "NA", "AZ": "AS", "BA": "EU", "BB": "NA",
    "BD": "AS", "BE": "EU", "BF": "AF", "BG": "EU", "BH": "AS", "BI": "AF", "BJ": "AF", "BM": "NA", "BN": "AS",
    "BO": "SA", "BR": "SA", "BS": "NA", "BT": "AS", "BW": "AF", "BY": "EU", "BZ": "NA", "CA": "NA", "CC": "AS",
    "CD": "AF", "CF": "AF", "CG": "AF", "CH": "EU", "CI": "AF", "CK": "OC", "CL": "SA", "CM": "AF", "CN": "AS",
    "CO": "SA", "CR": "NA", "CU": "NA", "CV": "AF", "CX": "AS", "CY": "AS", "CZ": "EU", "DE": "EU", "DJ": "AF",
    "DK": "EU", "DM": "NA", "DO": "NA", "DZ": "AF", "EC": "SA", "EE": "EU", "EG": "AF", "EH": "AF", "ER": "AF",
    "ES": "EU", "ET": "AF", "FI": "EU", "FJ": "OC", "FK": "SA", "FM": "OC", "FO": "EU", "FR": "EU", "GA": "AF",
    "GB": "EU", "GD": "NA", "GE": "AS", "GF": "SA", "GG": "EU", "GH": "AF", "GI": "EU", "GL": "NA", "GM": "AF",
    "GN": "AF", "GP": "NA", "GQ": "AF", "GR": "EU", "GS": "AQ", "GT": "NA", "GU": "OC", "GW": "AF", "GY": "SA",
    "HK": "AS", "HN": "NA", "HR": "EU", "HT": "NA", "HU": "EU", "ID": "AS", "IE": "EU", "IL": "AS", "IM": "EU",
    "IN": "AS", "IO": "AS", "IQ": "AS", "IR": "AS", "IS": "EU", "IT": "EU", "JE": "EU", "JM": "NA", "JO": "AS",
    "JP": "AS", "KE": "AF", "KG": "AS", "KH": "AS", "KI": "OC", "KM": "AF", "KN": "NA", "KP": "AS", "KR": "AS",
    "KW": "AS", "KY": "NA", "KZ": "AS", "LA": "AS", "LB": "AS", "LC": "NA", "LI": "EU", "LK": "AS", "LR": "AF",
    "LS": "AF", "LT": "EU", "LU": "EU", "LV": "EU", "LY": "AF", "MA": "AF", "MC": "EU", "MD": "EU", "ME": "EU",
    "MG": "AF", "MH": "OC", "MK": "EU", "ML": "AF", "MM": "AS", "MN": "AS", "MO": "AS", "MP": "OC", "MQ": "NA",
    "MR": "AF", "MS": "NA", "MT": "EU", "MU": "AF", "MV": "AS", "MW": "AF", "MX": "NA", "MY": "AS", "MZ": "AF",
    "NA": "AF", "NC": "OC", "NE": "AF", "NF": "OC", "NG": "AF", "NI": "NA", "NL": "EU", "NO": "EU", "NP": "AS",
    "NR": "OC", "NU": "OC", "NZ": "OC", "OM": "AS", "PA": "NA", "PE": "SA", "PF": "OC", "PG": "OC", "PH": "AS",
    "PK": "AS", "PL": "EU", "PM": "NA", "PN": "OC", "PR": "NA", "PS": "AS", "PT": "EU", "PW": "OC", "PY": "SA",
    "QA": "AS", "RE": "AF", "RO": "EU", "RS": "EU", "RU": "EU", "RW": "AF", "SA": "AS", "SB": "OC", "SC": "AF",
    "SD": "AF", "SE": "EU", "SG": "AS", "SH": "AF", "SI": "EU", "SJ": "EU", "SK": "EU", "SL": "AF", "SM": "EU",
    "SN": "AF", "SO": "AF", "SR": "SA", "ST": "AF", "SV": "NA", "SY": "AS", "SZ": "AF", "TC": "NA", "TD": "AF",
    "TF": "AQ", "TG": "AF", "TH": "AS", "TJ": "AS", "TK": "OC", "TM": "AS", "TN": "AF", "TO": "OC", "TR": "AS",
    "TT": "NA", "TV": "OC", "TW": "AS", "TZ": "AF", "UA": "EU", "UG": "AF", "US": "NA", "UY": "SA", "UZ": "AS",
    "VC": "NA", "VE": "SA", "VG": "NA", "VI": "NA", "VN": "AS", "VU": "OC", "WF": "OC", "WS": "OC", "YE": "AS",
    "YT": "AF", "ZA": "AF", "ZM": "AF", "ZW": "AF"}


def arguments():
    """
    Argument parser for command line usage.
    """
    global plc_host
    parser = argparse.ArgumentParser(
        prog="planetlab_list_creator",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description="Plb list creator is script, which will return you file with all available nodes from"
                    " www.planet-lab.eu. \nRequired arguments: USERNAME, PASSWORD and OUTPUT.")
    parser.add_argument("-u", "--username", required=True)
    parser.add_argument("-p", "--password", required=True)
    parser.add_argument("-o", "--output", required=True, action="store", help="Path where you want to save output.")
    parser.add_argument("--start_id", type=int, help="ID for the first node")
    parser.add_argument("-q", "--quiet", action="store_true", default=False, help="Suppress stdout")
    args, unknown_argument = parser.parse_known_args()
    if args.username:
        auth["Username"] = args.username
    if args.password:
        auth["AuthString"] = args.password
    if args.output:
        if not os.path.isdir(args.output):
            print("Directory doesn't exist.")
            sys.exit(1)
        arg["path"] = args.output

    if args.start_id:
        arg["id"] = args.start_id
        arg["start_id"] = args.start_id

    if args.quiet:
        arg["quiet"] = True

    if unknown_argument:
        print("Unknown argument")
        sys.exit(1)


def get_continent(country_code):
    """
    Function returns continent based on country code provided as parameter.\n
    :param country_code: string containing country code\n
    :return: None if country code is not recognized. Otherwise it will return continent code.\n
    """
    global continents
    if country_code is None:
        return None
    if country_code.upper() in list(continents.keys()):
        return continents[country_code.upper()]
    return None


def get_ip_address(hostname):
    """
    Function translates hostname to IP address.\n
    :param hostname: HOSTNAME\n
    :return: if hostname cannot be translated to IP address returns None. Otherwise it returns IP address as a string.
    """
    if hostname is not None or hostname is not '':
        try:
            ip_addr = socket.gethostbyname(hostname)
            return ip_addr
        except socket.error:
            pass
    return None


def append_to_file(node):
    """
    Function for writing information about a node into the file\n
    :param node:  dictionary containing information about a node
    """
    if arg["id"] == arg["start_id"]:
        try:
            f = open(os.path.join(arg["path"] + '/database/default2.node'), "w")
            f.write("# ID\tIP\tDNS\tCONTINENT\tCOUNTRY\tREGION\tCITY\tURL\tFULL NAME\tLATITUDE\tLONGITUDE\n")
            f.close()
        except Exception as err:
            traceback.print_exc()
            print(err)
            sys.exit(1)

    try:
        with codecs.open(os.path.join(arg["path"] + '/database/default2.node'),
                         mode="a", encoding="utf-8") as f:
            f.write("{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\n".format(str(arg["id"]), node["ip"],
                                                                          node["hostname"],
                                                                          node["continent"], node["country"],
                                                                          node["region"],
                                                                          node["city"], node["url"], node["name"],
                                                                          node["latitude"], node["longitude"]))
    except Exception as err:
        traceback.print_exc()
        print(err)
        sys.exit(1)


def print_info(node):
    """
    Function prints information about a node.\n
    :param node:  dictionary containing information about the node
    """
    print("ID: %s \n"
          "IP ADDRESS: \t%s\n"
          "HOSTNAME: \t%s\n"
          "CONTINENT: \t%s\n"
          "COUNTRY: \t%s\n"
          "REGION: \t%s\n"
          "CITY: \t\t%s\n"
          "URL: \t\t%s\n"
          "NAME: \t\t%s\n"
          "LATITUDE: \t%s\n"
          "LONGITUDE: \t%s\n\n"
          % (arg["id"], node["ip"], node["hostname"], node["continent"], node["country"], node["region"],
             node["city"], node["url"], node["name"], node["latitude"], node["longitude"]))


def run(path=None, username=None, password=None, start_id=None, quiet=False, return_output=False):
    """
    Function creates output file with all information about the nodes. It takes
    username and password from Planetlab web to authenticate for the API. In path param
    you specify the path where output shall be saved. These parameters are required and
    function will not work properly without them. Start id is optional and you should
    use it only if you want to use different numbering for nodes. Default value for
    start id is '1'. Quiet is boolean value for printing standard output.\n
    :param path: path to file as string. REQUIRED\n
    :param username: username of your Planetlab web account. REQUIRED\n
    :param password: password of your Planetlab web account. REQUIRED\n
    :param start_id: integer value for numbering. Optional: default=1\n
    :param quiet:  boolean value deciding whether to print information as standard output. Optional: default=False\n
    :param return_output: boolean value deciding whether to return output as list of nodes. Optional: default=False\n
    :return: lib/default2.node file, which contains following information about node:\n
        - ID\n
        - IP address\n
        - Hostname\n
        - Continent\n
        - Country\n
        - Region\n
        - City\n
        - URL\n
        - Name of an institution, which owns the node\n
        - Latitude\n
        - Longitude\n
    """
    global auth

    if path and username and password:
        auth["AuthString"] = password
        auth["Username"] = username
        arg["path"] = path
        if start_id:
            arg[start_id] = start_id
        if quiet:
            arg["quiet"] = quiet
    try:
        all_nodes = plc_api.GetNodes(auth, {"-SORT": "node_id"}, ["site_id", "hostname", "interface_ids"])
        for node in all_nodes:
            if len(node["interface_ids"]):
                interfaces = plc_api.GetInterfaces(auth, {"interface_id": node["interface_ids"][0]}, ["ip"])
                if len(interfaces) > 0:
                    node.update(interfaces[0])
                else:
                    node.update({"ip": "unknown"})
            else:
                addr = get_ip_address(node["hostname"])
                node["ip"] = addr

            node.update(plc_api.GetSites(auth, node["site_id"], ["latitude", "longitude", "url", "name"])[0])

            if node["site_id"] in list(locations.keys()):
                location = locations[node["site_id"]]
                node.update(
                    {"city": location[0], "region": location[1], "country": location[2], "continent": location[3]})
            else:
                sleep(1)  # Usage policy https://operations.osmfoundation.org/policies/nominatim/
                if node["latitude"] is not None and node["longitude"] is not None:
                    g = geocoder.osm([node["latitude"], node["longitude"]], method="reverse")
                    node["city"] = g.city
                    node["region"] = g.county
                    node["country"] = "unknown"
                    node["continent"] = "unknown"
                    if g.country_code is not None:
                        node["country"] = g.country_code.upper()
                        node["continent"] = get_continent(g.country_code)
                    locations.update(
                        {node["site_id"]: [node["city"], node["region"], node["country"], node["continent"]]})
                else:
                    node.update(
                        {"city": "unknown", "region": "unknown", "country": "unknown", "continent": "unknown"})

            # if value is None it is changed to unknown in info dictionary about node
            for value in list(node.values()):
                if value is None:
                    node[list(node.keys())[list(node.values()).index(value)]] = "unknown"

            append_to_file(node)
            if not arg["quiet"]:
                print_info(node)
            arg["id"] += 1

    except (KeyboardInterrupt, SystemExit):
        print("Program stopped by user. Exiting....")
        sys.exit(1)

    except Exception as err:
        traceback.print_exc()
        print(err)
        sys.exit(1)

    if return_output:
        return all_nodes

#TODO: musis dodelat predavani parametru
def setLocalVariables():
    print("TBD")

def listCreatorMain():
    print("Start: %s" % asctime(localtime(time())))
    arguments()
    run()
    print("Finish: %s" % asctime(localtime(time())))

if __name__ == "__main__":
    print("Start: %s" % asctime(localtime(time())))
    arguments()
    run()
    print("Finish: %s" % asctime(localtime(time())))




#! /usr/bin/env python3
# Author: Martin Kacmarcik
#License: MIT
# For my Diploma thesis at Faculty of Electrical Engineering -- Brno, University of Technology

import webbrowser
import folium
import subprocess
import signal
import re
import locale
from dialog import Dialog
import hashlib
import sqlite3
from platform import system
from multiprocessing import Pool, Lock, Value
import sys
import os
import csv


# local imports
sys.path.insert(0, os.path.dirname(os.path.realpath(__file__)))
from lib import full_map
from lib import port_scanner


# Constant definition
OPTION_LOCATION = 0
OPTION_IP = 1
OPTION_DNS = 2
OPTION_CONTINENT = 3
OPTION_COUNTRY = 4
OPTION_REGION = 5
OPTION_CITY = 6
OPTION_URL = 7
OPTION_NAME = 8
OPTION_LAT = 9
OPTION_LON = 10
VERSION = "0.3.3"

#global variables
base = None
increment = None
path = ""

# Initial settings
locale.setlocale(locale.LC_ALL, '')
d = Dialog(dialog="dialog")
d.set_background_title("Planetlab Server Manager "+VERSION)


def signal_handler(sig, frame):
    clear()
    print('Terminating program. You have pressed Ctrl+C')
    exit(1)


def getPath():
    global path
    os.chdir(os.path.dirname(os.path.realpath(__file__)))
    path = os.path.dirname(os.path.realpath(__file__))
    return path


def clear():
    os.system("clear")


def testPing(target, returnbool=False):
    pingPacketWaitTime = None
    if system().lower() == 'windows':
        pingParam = '-n'
    else:
        pingParam = '-c'
    #for Linux ping parameter takes seconds while MAC OS ping takes miliseconds
    if system().lower() == 'linux':
        pingPacketWaitTime = 1
    else:
        pingPacketWaitTime = 800
    command = ['ping', pingParam, '1', target, '-W', str(pingPacketWaitTime)]
    p = subprocess.Popen(command, stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE)
    # prepare the regular expression to get time
    if system().lower() == 'windows':
        avg = re.compile('Average = ([0-9]+)ms')
    else:
        avg = re.compile(
            'min/avg/max/[a-z]+ = [0-9.]+/([0-9.]+)/[0-9.]+/[0-9.]+')
    avgStr = avg.findall(str(p.communicate()[0]))
    if p.returncode != 0:
        if not returnbool:
            return "Not reachable via ICMP"
        return False
    else:
        p.kill()
        if not returnbool:
            return avgStr[0]+" ms"
        return True


def testSsh(target, printwarn=True):
    result = port_scanner.testPortAvailability(target, 22)
    if (result is True or result is False):
        return result
    elif result is 98:
        if printwarn is True:
            d.msgbox("Hostname could not be resolved. Either the server has been removed, \
                     your network is not working or you have wrongly set DNS.", height=0, width=0)
        return result
    elif result is 97:
        if printwarn is True:
            d.msgbox(
                "Error while connecting to server. Please check your network settings.")
        return result


def verifyApiCredentialsExist():
    with open(path+"/conf/plbmng.conf", 'r') as config:
        for line in config:
            if re.search('USERNAME', line):
                username = re.sub('USERNAME="(.*)"', r'\1', line).rstrip()
                if not username:
                    return False
            elif re.search('PASSWORD', line):
                password = re.sub('PASSWORD="(.*)"', r'\1', line).rstrip()
                if not password:
                    return False
    return True


def verifySshCredentialsExist():
    with open(path+"/conf/plbmng.conf", 'r') as config:
        for line in config:
            if re.search('SLICE', line):
                planetlab_slice = re.sub('SLICE="(.*)"', r'\1', line).rstrip()
                if not planetlab_slice:
                    return False
            elif re.search('SSH_KEY', line):
                key = re.sub('SSH_KEY="(.*)"', r'\1', line).rstrip()
                if not key:
                    return False
    return True

# parent function to update availability DB that controls multiple processes


def updateAvailabilityDatabaseParent(mode=None):
    if mode == 'cron':
        path = getPath()
    nodes = getNodes(False)
    increment = Value('f', 0)
    increment.value = float(100/len(nodes))
    base = Value('f', 0)
    lock = Lock()
    d.gauge_start()
    pool = Pool(50, initializer=multiProcessingInit,
                initargs=(lock, base, increment, ))
    pool.map(updateAvailabilityDatabase, nodes)
    pool.close()
    pool.join()
    d.gauge_update(100, "Completed")
    exit_code = d.gauge_stop()
    d.msgbox("Availability database has been successfully updated")


def multiProcessingInit(l, b, i):
    global lock
    lock = l
    global base
    base = b
    global increment
    increment = i


def updateProgressBarMultiProcessing(increment_number):
    global d
    d.gauge_update(int(increment_number))


def updateAvailabilityDatabase(node):
    # inint block
    db = sqlite3.connect('database/internal.db')
    cursor = db.cursor()
    # action block
    hash_object = hashlib.md5(node[2].encode())
    ssh_result = 'T' if testSsh(node[2], False) == True else 'F'
    ping_result = 'T' if testPing(node[1], True) == True else 'F'
    # find if object exists in the database
    cursor.execute('SELECT nkey from AVAILABILITY where \
            shash = \"'+str(hash_object.hexdigest())+'\";')
    if cursor.fetchone() is None:
        # the object yet doesn't exist
        cursor.execute("INSERT into AVAILABILITY(shash, shostname, bssh, bping) VALUES\
                        (\""+hash_object.hexdigest()+"\", \""+node[2]+"\",\
                        \""+ssh_result+"\", \""+ping_result+"\")")
    else:
        # object exists, just update results
        cursor.execute("UPDATE availability SET bssh=\""+ssh_result+"\", bping=\"" +
                       ping_result+"\" WHERE shash=\""+hash_object.hexdigest()+"\"")
    lock.acquire()
    base.value = base.value + increment.value
    updateProgressBarMultiProcessing(base.value)
    lock.release()
    # clean up
    db.commit()
    db.close()


def getSshKey():
    sshPath = ""
    with open(path+"/conf/plbmng.conf", 'r') as config:
        for line in config:
            if re.search('SSH_KEY', line):
                sshPath = (re.sub('SSH_KEY=', '', line)).rstrip()
    return sshPath


def getSshUser():
    user = ""
    with open(path+"/conf/plbmng.conf", 'r') as config:
        for line in config:
            if re.search('SLICE=', line):
                user = (re.sub('SLICE=', '', line)).rstrip()
    return user


def getUser():
    user = ""
    with open(path+"/conf/plbmng.conf", 'r') as config:
        for line in config:
            if re.search('USERNAME=', line):
                user = (re.sub('USERNAME=', '', line)).rstrip()
    return user


def getPasswd():
    passwd = ""
    with open(path+"/conf/plbmng.conf", 'r') as config:
        for line in config:
            if re.search('PASSWORD=', line):
                passwd = (re.sub('PASSWORD=', '', line)).rstrip()
    return passwd


def getNodes(checkConfiguration=True, chooseAvailabilityOption=None):
    # Initialize filtering settings
    db = sqlite3.connect('database/internal.db')
    cursor = db.cursor()
    if chooseAvailabilityOption is None:
        cursor.execute('SELECT * from configuration where senabled=\'T\';')
        configuration = cursor.fetchall()
        if not configuration:
            sql = 'select shostname from availability'
        else:
            sql = 'select shostname from availability where'
            for item in configuration:
                if re.match(r'.*where$', sql):
                    sql = sql + ' b'+item[1]+'=\'T\''
                else:
                    sql = sql + ' and b'+item[1]+'=\'T\''
    elif chooseAvailabilityOption == 1:
        sql = "select shostname from availability where bping='T'"
    elif chooseAvailabilityOption == 2:
        sql = "select shostname from availability where bssh='T'"
    elif chooseAvailabilityOption == 3:
        sql = "select shostname from availability"
    cursor.execute(sql)
    returnedValuesSql = cursor.fetchall()
    # convert returned tuples to list
    serverList = []
    for item in returnedValuesSql:
        serverList.append(item[0])
    # open node file and append to the nodes if the element exists in the serverList
    nodeFile = path+'/database/default.node'
    nodes = []
    with open(nodeFile) as tsv:
        for line in csv.reader(tsv, delimiter='\t'):
            if line[0] == '# ID':
                continue
            try:
                if checkConfiguration:
                    if line[2] in serverList:
                        nodes.append(line)
                else:
                    nodes.append(line)
            except ValueError:
                pass
    # clean up block
    db.close()
    return nodes


def getAllNodes():
    user = getUser()
    passwd = getPasswd()
    if user != "" and passwd != "":
        os.system("myPwd=$(pwd); cd "+path+"; python3 lib/planetlab_list_creator.py -u \"" +
                  user+"\" -p \""+passwd+"\" -o ./; cd $(echo $myPwd)")
    else:
        needToFillPasswdFirstInfo()


def isFirstRun():
    isFirst = path+'/database/first.boolean'
    with open(isFirst, 'r') as isFirstFile:
        bIsFirst = isFirstFile.read().strip('\n')
    if bIsFirst == "True":
        with open(isFirst, 'w') as isFirstFile:
            isFirstFile.write("False")
        return True
    else:
        return False

def getFiltersForAccessServers():
    sshFilter=None
    pingFilter=None
    db = sqlite3.connect('database/internal.db')
    cursor = db.cursor()
    cursor.execute('SELECT * from configuration;')
    configuration = cursor.fetchall()
    for item in configuration:
        if item[1] == 'ssh':
            if item[2] == 'T':
                sshFilter=True
            else:
                sshFilter=False
        elif item[1] == 'ping':
            if item[2] == 'T':
                pingFilter=True
            else:
                pingFilter=False
    if sshFilter and pingFilter:
        filterOptions = "Only SSH and ICMP available"
    elif sshFilter and not pingFilter:
        filterOptions = "Only SSH available"
    elif not sshFilter and pingFilter:
        filterOptions = "Only PING available"
    else:
        filterOptions = "None"
    return filterOptions

def lastServerMenu():
    infoAboutNodeDic, chosenNode = getLastServerAccess()
    if infoAboutNodeDic == None:
        return
    returnedChoice = printServerInfo(infoAboutNodeDic)
    if returnedChoice == None:
        return
    elif not returnedChoice:
        return
    elif int(returnedChoice) == 1:
        connect(int(returnedChoice), chosenNode)
    elif int(returnedChoice) == 2:
        connect(int(returnedChoice), chosenNode)
    elif int(returnedChoice) == 3:
        showOnMap(chosenNode, infoAboutNodeDic)


def updateLastServerAccess(infoAboutNodeDic, chosenNode):
    lastServerFile = path+'/database/last_server.node'
    with open(lastServerFile, 'w') as lastServerFile:
        lastServerFile.write(repr((infoAboutNodeDic, chosenNode)))


def getLastServerAccess():
    lastServerFile = path+'/database/last_server.node'
    try:
        with open(lastServerFile, 'r') as lastServerFile:
            infoAboutNodeDic, chosenNode = eval(lastServerFile.read().strip('\n'))
        return infoAboutNodeDic, chosenNode
    except FileNotFoundError:
        d.msgbox("You did not access any server yet.")
        return None, None


def searchNodes(option, regex=None):
    nodes = getNodes()
    answers = []
    choices = []
    counter = 1
    # non-LOCATION search
    if option is not OPTION_LOCATION:
        for item in nodes:
            if re.search(regex, item[option]):
                answers.append(item)
        if len(answers) == 0:
            returnedChoice = searchNodesGui(False)
        else:
            # prepare choices for GUI
            for item in answers:
                choices.append((str(counter), item[option]))
                counter += 1
            returnedChoice = searchNodesGui(choices)
    # LOCATION based search
    else:
        for item in nodes:
            answers.append(item[OPTION_CONTINENT])
        continents = sorted(set(answers))
        # prepare choices for GUI
        for item in continents:
            choices.append((str(counter), item))
            counter += 1
        returnedChoice = searchNodesGui(choices)
        if returnedChoice == None:
            return
        answers = []
        choices = []
        counter = 1
        for item in nodes:
            if re.search(continents[int(returnedChoice)-1], item[OPTION_CONTINENT]):
                answers.append(item[OPTION_COUNTRY])
        countries = sorted(set(answers))
        # prepare choices for GUI
        for item in countries:
            choices.append((str(counter), item))
            counter += 1
        returnedChoice = searchNodesGui(choices)
        if returnedChoice == None:
            return
        answers = []
        choices = []
        counter = 1
        for item in nodes:
            if re.search(countries[int(returnedChoice)-1], item[OPTION_COUNTRY]):
                answers.append(item[OPTION_DNS])
        hostnames = sorted(set(answers))
        # prepare choices for GUI
        for item in hostnames:
            choices.append((str(counter), item))
            counter += 1
        returnedChoice = searchNodesGui(choices)
    if returnedChoice == None:
        return
    else:
        returnedChoice, infoAboutNodeDic, chosenNode = getServerInfo(
            choices[int(returnedChoice)-1][1], option, nodes)
    if returnedChoice == None:
        return
    elif not returnedChoice:
        return
    elif int(returnedChoice) == 1:
        connect(int(returnedChoice), chosenNode)
    elif int(returnedChoice) == 2:
        connect(int(returnedChoice), chosenNode)
    elif int(returnedChoice) == 3:
        showOnMap(chosenNode, infoAboutNodeDic)


def connect(mode, node):
    clear()
    key = getSshKey()
    user = getSshUser()
    if mode == 1:
        return_value = os.system(
            "ssh -o \"StrictHostKeyChecking = no\" -o \"UserKnownHostsFile=\
            /dev/null\" -i "+key+" "+user+"@"+node[OPTION_IP])
        if return_value is not 0:
            d.msgbox("Error while connecting. Please verify your credentials.")
    elif mode == 2:
        os.system('ssh-add '+key)
        return_value = os.system("mc sh://"+user+"@"+node[OPTION_IP]+":/home")
        if return_value is not 0:
            d.msgbox("Error while connecting. Please verify your credentials.")
    else:
        return


def showOnMap(node, nodeInfo=""):
    latitude = float(node[-2])
    longitude = float(node[-1])
    name = node[OPTION_DNS]
    popup = folium.Popup(nodeInfo["text"].strip().replace('\n', '<br>'), max_width=1000)
    nodeMap = folium.Map(location=[latitude, longitude],
                         zoom_start=2, min_zoom=2)
    if nodeInfo == "":
        folium.Marker([latitude, longitude], popup=popup).add_to(nodeMap)
    else:
        folium.Marker([latitude, longitude], popup).add_to(nodeMap)
    nodeMap.save('/tmp/map_plbmng.html')
    webbrowser.get().open('file://' + os.path.realpath('/tmp/map_plbmng.html'))

def plotServersOnMap(mode):
    # update base_data.txt file based on latest database with nodes
    nodes = getNodes(True, int(mode))
    full_map.plot_server_on_map(nodes)
    mapFile = "plbmng_server_map.html"
    webbrowser.get().open('file://' + os.path.realpath(path+"/"+mapFile))


def getServerInfo(serverId, option, nodes=None):
    if nodes == None:
        nodes = getNodes()
    if option == 0:
        option = OPTION_DNS
    if isinstance(serverId, str):
            # in nodes find the chosenOne node
        chosenOne = ""
        for item in nodes:
            if re.search(serverId, item[option]):
                chosenOne = item
                break
        if chosenOne == "":
            print("Internal error, please file a bug report via PyPi")
            exit(99)
        # get information about servers
        infoAboutNodeDic = dict()
        region, city, url, fullname, lat, lon = getInfoFromNode(chosenOne)
        infoAboutNodeDic["region"] = region
        infoAboutNodeDic["city"] = city
        infoAboutNodeDic["url"] = url
        infoAboutNodeDic["fullname"] = fullname
        infoAboutNodeDic["lat"] = lat
        infoAboutNodeDic["lon"] = lon
        infoAboutNodeDic["icmp"] = testPing(chosenOne[OPTION_IP])
        infoAboutNodeDic["sshAvailable"] = testSsh(chosenOne[OPTION_DNS])
        infoAboutNodeDic["text"] = """
            NODE: %s
            IP: %s
            CONTINENT: %s, COUNTRY: %s, REGION: %s, CITY: %s
            URL: %s
            FULL NAME: %s
            LATITUDE: %s, LONGITUDE: %s
            CURRENT ICMP RESPOND: %s
            CURRENT SSH AVAILABILITY: %r
            """ % (chosenOne[OPTION_DNS],
                   chosenOne[OPTION_IP],
                   chosenOne[OPTION_CONTINENT],
                   chosenOne[OPTION_COUNTRY],
                   infoAboutNodeDic["region"],
                   infoAboutNodeDic["city"],
                   infoAboutNodeDic["url"],
                   infoAboutNodeDic["fullname"],
                   infoAboutNodeDic["lat"],
                   infoAboutNodeDic["lon"],
                   infoAboutNodeDic["icmp"],
                   infoAboutNodeDic["sshAvailable"])
        if infoAboutNodeDic["sshAvailable"] is True or infoAboutNodeDic["sshAvailable"] is False:
            # update last server access database
            updateLastServerAccess(infoAboutNodeDic, chosenOne)
            return printServerInfo(infoAboutNodeDic), infoAboutNodeDic, chosenOne
        else:
            return False, False, False


def getInfoFromNode(node):
    region = node[5]
    city = node[6]
    url = node[7]
    fullname = node[8]
    lat = node[9]
    lon = node[10]
    return region, city, url, fullname, lat, lon


def removeCron():
    os.system("crontab -l | grep -v \"plbmng crontab\" | crontab -")


def addToCron(mode):
    if int(mode) == 2:
        line = "@daily plbmng crontab"
    elif int(mode) == 3:
        line = "@weekly plbmng crontab"
    elif int(mode) == 4:
        line = "@monthly plbmng crontab"
    os.system("echo \"$(crontab -l ; echo "+line+")\" | crontab -")

###################
#  GUI functions  #
###################


def printServerInfo(infoAboutNodeDic):
    if not verifySshCredentialsExist():
        preparedChoices = [("1", "Connect via SSH (Credentials not set!)"),
                           ("2", "Connect via MC (Credentials not set!)"),
                           ("3", "Show on map")]
    else:
        preparedChoices = [("1", "Connect via SSH"),
                           ("2", "Connect via MC"),
                           ("3", "Show on map")]
    code, tag = d.menu(
        infoAboutNodeDic["text"], height=0, width=0, menu_height=0, choices=preparedChoices)
    if code == d.OK:
        return tag
    else:
        return None


def searchNodesGui(prepared_choices):
    if not prepared_choices:
        d.msgbox("No results found.", width=0, height=0)
        return None
    while True:
        code, tag = d.menu("These are the results:",
                           choices=prepared_choices,
                           title="Search results")
        if code == d.OK:
            return tag
        else:
            return None


def firstRunMessage():
    d.msgbox("This is your first run of the application. Please go to 'Set Credentials' menu and set your credentials now.", height=0, width=0)


def needToFillPasswdFirstInfo():
    d.msgbox("Credentials are not set. Please go to menu and set them now")


def initInterface():
    getPath()
    signal.signal(signal.SIGINT, signal_handler)
    if isFirstRun():
        firstRunMessage()
    while True:
        # Main menu
        code, tag = d.menu("Choose one of the following options:",
                           choices=[("1", "Access servers"),
                                    ("2", "Monitor servers"),
                                    ("3", "Plot servers on map"),
                                    ("4", "Set credentials"),
                                    ("5", "About")],
                           title="MAIN MENU")

        if code == d.OK:
            # Acess servers
            if tag == "1":
                accessServersGui()
            # Meausre servers
            elif tag == "2":
                monitorServersGui()
            # Plot servers on map
            elif tag == "3":
                plotServersOnMapGui()
            # Set crdentials
            elif tag == "4":
                setCredentialasGui()
            elif tag == "5":
                aboutGui()
        else:
            clear()
            exit(0)


def setCredentialasGui():
    code, text = d.editbox(path+'/conf/plbmng.conf', height=0, width=0)
    if code == d.OK:
        with open(path+'/conf/plbmng.conf', "w") as configFile:
            configFile.write(text)


def filteringOptionsGui():
    # inint the database to pull configuration data
    db = sqlite3.connect('database/internal.db')
    cursor = db.cursor()
    cursor.execute("select senabled from configuration where sname='ssh'")
    ssh_enabled = True if cursor.fetchone()[0] == 'T' else False
    cursor.execute("select senabled from configuration where sname='ping'")
    ping_enabled = True if cursor.fetchone()[0] == 'T' else False
    # Render the GUI
    code, t = d.checklist(
        "Press SPACE key to choose filtering options", height=0, width=0, list_height=0,
        choices=[("1", "Enable SSH accessible machines", ssh_enabled),
                 ("2", "Enable PING accessible machines", ping_enabled)], )
    # Detect changes using simple combination and update the database based on result
    if '1' in t and not ssh_enabled:
        cursor.execute(
            'UPDATE configuration SET senabled="T" where sname="ssh"')
        db.commit()
    elif '1' not in t and ssh_enabled:
        cursor.execute(
            'UPDATE configuration SET senabled="F" where sname="ssh"')
        db.commit()
    if '2' in t and not ping_enabled:
        cursor.execute(
            'UPDATE configuration SET senabled="T" where sname="ping"')
        db.commit()
    elif '2' not in t and ping_enabled:
        cursor.execute(
            'UPDATE configuration SET senabled="F" where sname="ping"')
        db.commit()
    # Clean up database
    db.close()


def aboutGui():
    d.msgbox("""
            PlanetLab Server Manager
            Project supervisor:
                Dan Komosny
            Authors:
                Tomas Andrasov
                Filip Suba
                Martin Kacmarcik

            Version """+VERSION+"""
            This application is under MIT license.
            """, width=0, height=0, title="About")

# Plot servers on map part of GUI


def plotServersOnMapGui():
    while True:
        code, tag = d.menu("Choose one of the following options:",
                           choices=[("1", "Plot servers reponding to ping"),
                                    ("2", "Plot ssh available servers"),
                                    ("3", "Plot all servers")],
                           title="Map menu")
        if code == d.OK:
            plotServersOnMap(tag)
            return
        else:
            return

# Monitor servers part of GUI


def monitorServersGui():
    if not verifyApiCredentialsExist():
        d.msgbox("Warning! Your credentials for PlanetLab API are not set. Please use 'Set credentials' option in main menu to set them.")
    while True:
        code, tag = d.menu("Choose one of the following options:",
                           choices=[("1", "Set crontab for status update"),
                                    ("2", "Update server list now"),
                                    ("3", "Update server status now")],
                           title="Monitoring menu", height=0, width=0)
        if code == d.OK:
            if tag == "1":
                code, tag = d.menu("Choose one of the following options:",
                                   choices=[("1", "Set monitoring daily"),
                                            ("2", "Set monitoring weekly"),
                                            ("3", "Set monitoring monthly"),
                                            ("4", "Remove all monitoring from cron")],
                                   title="Crontab menu")
                if code == d.OK:
                    if tag == "1":
                        addToCron(tag)
                    elif tag == "2":
                        addToCron(tag)
                    elif tag == "3":
                        addToCron(tag)
                    elif tag == "4":
                        removeCron()
                else:
                    continue
            elif tag == "2":
                if d.yesno("This is going to take around 20 minutes") == d.OK:
                    getAllNodes()
                else:
                    continue
            elif tag == "3":
                if d.yesno("This can take few minutes. Do you want to continue?") == d.OK:
                    if not verifySshCredentialsExist():
                        d.msgbox(
                            "Error! Your ssh credentials are not set. Please use 'Set credentials' option in main menu to set them.")
                        continue
                    else:
                        updateAvailabilityDatabaseParent()
                else:
                    continue
        else:
            return


# Acess servers part of GUI
def accessServersGui():
    while True:
        filterOptions = getFiltersForAccessServers()
        menuText= """
        \nActive filters: """+filterOptions

        code, tag = d.menu("Choose one of the following options:"+menuText,
                           choices=[
                                    ("1", "Access last server"),
                                    ("2", "Serach by DNS"),
                                    ("3", "Search by IP"),
                                    ("4", "Search by location"),
                                    ("5", "Filtering options")],
                           title="ACCESS SERVERS")
        if code == d.OK:
            # Search by DNS
            if tag == "2":
                code, answer = d.inputbox("Search for:"+menuText, title="Search", width=0, height=0)
                if code == d.OK:
                    searchNodes(OPTION_DNS, answer)
                else:
                    continue
            # Search by IP
            elif tag == "3":
                code, answer = d.inputbox("Search for:"+menuText, title="Search", width=0, height=0)
                if code == d.OK:
                    searchNodes(OPTION_IP, answer)
                else:
                    continue
            # Search by location
            elif tag == "4":
                # Grepuje se default node
                searchNodes(OPTION_LOCATION)
            # Access last server
            elif tag == "1":
                lastServerMenu()
            # Filtering options
            elif tag == "5":
                filteringOptionsGui()

        else:
            return


if __name__ == "__main__":
    initInterface()
    exit(0)



import os
import sys
from setuptools import setup, find_packages

install_requires = ['vincent', 'folium', 'numpy', 'pandas', 'geocoder', 'paramiko', 'pythondialog']

setup(name='plbmng',
      description='Tool for monitoring PlanetLab network',
      version="0.3.3",
      license='MIT',
      packages=find_packages(),
      package_data={},
      include_package_data=True,
      install_requires=install_requires,
      dependency_links=['https://github.com/pandas-dev/pandas/archive/master.zip?ref=master#egg=pandas'],
      url='https://github.com/Andrasov/PlanetLab.git',
      author='Ivan Andrasov',
      author_email='xandra03@stud.feec.vutbr.cz',
      # maintainer='Dan Komosny',
      # maintainer_email='komosny@feec.vutbr.cz'
      long_description=open("README.rst").read(),
      scripts=['bin/plbmng']
      )
